@@ -1,24 +1,27 @@
 from .plot import Plot
-from .tools import show_base, save_base, is_color, format_color
+from .tools import show_base, save_base, is_color, format_color, is_iterable
 from ..tools.color_palettes import palette_from_name_to_function
 
 import numbers
 
+from bokeh.models import Range1d
 from bokeh.plotting import figure
 from bokeh.models import ColorBar, LinearColorMapper
 import numpy as np
 import pandas as pd
 
 
-def line_base(y, x, source_dataframe, width, height, description, title, x_label, y_label,
-              show_plot, color, colorbar_type, legend, line_width, alpha, style,
-              x_axis_type, y_axis_type, fill_between, grid_visible, session,
-              save_path):
-    """ One dimensional plot
+def line_base(y, x, source_dataframe, width, height, description, title,
+              x_label, y_label, show_plot, color, colorbar_type, legend,
+              line_width, alpha, style, x_axis_type, y_axis_type, x_range,
+              y_range, fill_between, grid_visible, session, save_path):
+    """ One dimensional plot. This is the docstring of line
 
     Args:
         x (array-like): X-axis data
         y (array-like): y-axis data
+        x_range (array-like) Range of the x-axis. E.g. `[0, 1]`
+        y_range (array-like) Range of the y-axis. E.g. `[0, 1]`
     """
     # We convert (source, x and y) into only x and y. x and y will be processed
     # normally. source will not be used any more.
@@ -125,15 +128,17 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
     # the 3 values are different). For that case we use the argument
     # `automatic_color_mapping` in from the session.
 
-    elif (len(color) == 3) and isinstance(color[0],
-                                          numbers.Real) and session.automatic_color_mapping and (
-            len(y) == 3):
+    elif (len(color) == 3) \
+            and isinstance(color[0], numbers.Real)\
+            and session.automatic_color_mapping\
+            and (len(y) == 3):
         color = [color, color, color]
     else:
         # General case
         if is_color(color):
             color = [color for _ in y]
-        elif isinstance(color, (list, np.ndarray, tuple)) and is_color(color[0]):
+        elif isinstance(color, (list, np.ndarray, tuple))\
+                and is_color(color[0]):
             if len(color) == len(y):
                 pass  # Good case
             elif len(color) == 2:
@@ -157,7 +162,7 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
 
             if colorbar_type.lower() == 'auto':
                 if ((len(np.unique(color)) / len(color)) <= 0.5) and (
-                len(np.unique(color) < 9)):
+                  len(np.unique(color) < 9)):
                     # Data are considered categorical
                     colorbar_type = 'categorical'
                 else:
@@ -166,7 +171,7 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
                 colorbar_type = colorbar_type.lower()
 
             if colorbar_type == 'categorical':
-                if legend.lower() == 'auto':
+                if isinstance(legend, str) and (legend.lower() == 'auto'):
                     legend = [str(c) for c in color]
                 color_unique = sorted(list(np.unique(color)))
                 nb_color_needed = len(color_unique)
@@ -256,10 +261,12 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
             except ValueError:
                 # TODO: Change message error can x[0][0] may not make sense for
                 # the user. Same for y axis below
-                raise ValueError('`x_axis_type` is set to `auto` and x[0][0] '
-                                 'is neither a number, nor an object parsable '
-                                 'as a date. Object x[0][0]: {}'.format(
-                    x[0][0]))
+                error_msg = (
+                    '`x_axis_type` is set to `auto` and x[0][0] '
+                    'is neither a number, nor an object parsable '
+                    'as a date. Object x[0][0]: {}'
+                ).format(x[0][0])
+                raise ValueError(error_msg)
             x_axis_type = 'datetime'
             x = [pd.to_datetime(x_i) for x_i in x]
     elif x_axis_type.lower() in ['numeric', 'numerical']:
@@ -275,10 +282,12 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
             try:
                 pd.to_datetime(y[0][0], errors='raise')
             except ValueError:
-                raise ValueError('`y_axis_type` is set to `auto` and y[0][0] '
-                                 'is neither a number, nor an object parsable '
-                                 'as a date. Object y[0][0]: {}'.format(
-                    y[0][0]))
+                error_msg = (
+                    '`y_axis_type` is set to `auto` and y[0][0] '
+                    'is neither a number, nor an object parsable '
+                    'as a date. Object y[0][0]: {}'
+                ).format(y[0][0])
+                raise ValueError(error_msg)
             y_axis_type = 'datetime'
             y = [pd.to_datetime(y_i) for y_i in y]
     elif y_axis_type.lower() in ['numeric', 'numerical']:
@@ -294,8 +303,9 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
                                                         style):
         if leg_i:
             legend_exist = True
+
             def step(f, x_copy=x_i, y_copy=y_i, col_c=col_i, leg_c=leg_i,
-                        lw_c=lw_i, a_c=a_i, s_c=s_i):
+                     lw_c=lw_i, a_c=a_i, s_c=s_i):
                 f.line(x=x_copy, y=y_copy, color=col_c, legend_label=leg_c,
                        line_width=lw_c, alpha=a_c, line_dash=s_c)
         else:
@@ -335,6 +345,14 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
         fig.xgrid.grid_line_dash = [8, 3, 2, 3]
         fig.ygrid.grid_line_dash = [8, 3, 2, 3]
         fig.toolbar.autohide = True
+        if x_range is not None:
+            if not is_iterable(x_range):
+                raise ValueError('`x_range` argument should be an iterable')
+            fig.x_range = Range1d(x_range[0], x_range[1])
+        if y_range is not None:
+            if not is_iterable(y_range):
+                raise ValueError('`y_range` argument should be an iterable')
+            fig.y_range = Range1d(y_range[0], y_range[1])
         if _color_bar_made and _add_color_bar:
             color_mapper = LinearColorMapper(palette=palette_colors,
                                              low=color_min,
@@ -357,22 +375,136 @@ def line_base(y, x, source_dataframe, width, height, description, title, x_label
     else:
         return plot
 
+
 def _update_line_default_args(line, session):
     def line_updated(y, x=None, source_dataframe=None, width=session.width,
                      height=session.height, description=session.description,
                      title=session.title, x_label=None, y_label=None,
                      show_plot=session.show_plot, color=None,
-                     colorbar_type='auto', legend='auto', line_width=1, alpha=1,
-                     style='solid', x_axis_type='auto', y_axis_type='auto',
+                     colorbar_type='auto', legend='auto', line_width=1,
+                     alpha=1, style='solid', x_axis_type='auto',
+                     y_axis_type='auto', x_range=None, y_range=None,
                      fill_between=False, save_path=session.save_path,
                      grid_visible=session.grid_visible):
+        """Plot a graph with one-dimensional line(s)
+
+        Args:
+            y (array-like of dimension 1 or 2): The y-coordinates for the
+                points of the line(s). If `y` is 1-d, one line will be drawn,
+                if  it is 2-d, a set of lines will be drawn.
+                The values can be either numbers, or dates (datetimes or
+                parsable strings). If a `source_dataframe` is given, `x` and
+                `y` should be column names / keys.
+
+            x (None, array-like of dimension 1 or 2): The x-coordinates for the
+                points of the line(s). If `None`, indexes starting from 0 and
+                adapted to `y` will be used. If `y` is 1-d, one line will be
+                drawn, if  it is 2-d, a set of lines will be drawn.
+                The values can be either numbers, or dates (datetimes or
+                parsable strings). If a `source_dataframe` is given, `x` and
+                `y` should be column names / keys.
+
+            source_dataframe (None, Pandas DataFrame or dict): Input data as
+                Pandas DataFrame or dictionary. If it is a Pandas DataFrame,
+                and `x` is None, the index of the dataframe is used as `x`
+
+            width (int): The width of the graph, including any axes, titles,
+                etc
+
+            height (int): The height of the graph, including any axes, titles,
+                etc
+
+            description (str): HTML-formatted text that will be kept bellow the
+                graph. It generally includes metadata, details about the,
+                graphs, etc. It will be kept when the graph is displayed,
+                exported or rendered in a grid with other graphs. If The graph
+                is summed with other graph, their metadata will be concatenated
+                (adding a new line between both)
+
+            title (str): Title of the graph. The attributes of the title
+                (font), cannot be tuned directly in depict
+
+            x_label (None, str): Label of the x-axis
+
+            y_label (None, str): Label of the y-axis
+
+            show_plot (bool): Whether or not the graph must be displayed
+                immediatly after its creation
+
+            color (None, str, array-like): If None, the first color of the
+                palette is used.
+                If string, a color name is expected (hexadecimal, RGB, and
+                usual colors are accepted), and it will be the same for all
+                the lines.
+                If array-like, the length of color should correspond to the
+                number of lines drawn, they will correspond to each line
+                respectively
+
+            colorbar_type ({`auto`, `categorical`, `continuous`}): If 'auto',
+                the best type will be chosen wrt the data. If `categorical`:
+                a legend will be used, not a colorbar. If `continuous`, a
+                colorbar will be displayed on the right side of the plot.
+                The data defining the color must be passed in `color`
+
+            legend ('auto', array-like): If `auto`, the legend will be set when
+                a Pandas DataFrame ora dictionary is provided (the name of the
+                columns / keys will be the legend). If array-like, the length
+                of `legend` must match with the number of curves dranw
+
+            line_width (Number, array-like): Width of the line(s). If Number
+                it will the same for all the lines. If it is an array-like,
+                its length must match with the number of lines drawn. They will
+                correspond to each line respectively
+
+            alpha (Number, array-like): Alpha value of the line(s). If Number
+                it will the same for all the lines. If it is an array-like,
+                its length must match with the number of lines drawn. They will
+                correspond to each line respectively
+
+            style ({'solid', 'dashed', 'dotted', 'dotdash', dashdot'} or array
+                like of those): Style of the line(s). If it is an array-like,
+                its length must match with the number of lines drawn. They will
+                correspond to each line respectively
+
+            x_axis_type ({'auto', 'numerical', 'datetime'}): Type of the axis.
+                If 'auto', the type will be set automatically based on the data
+                provided. If 'numerical', 'datetime', the type is set
+                accordingly
+
+            y_axis_type ({'auto', 'numerical', 'datetime'}): Type of the axis.
+                If 'auto', the type will be set automatically based on the data
+                provided. If 'numerical', 'datetime', the type is set
+                accordingly
+
+            x_range (array-like): Range of the x-axis
+
+            y_range (array-like): Range of the y-axis
+
+            fill_between (bool): If True, there must be at least 2 curves to
+                plot and the area between the first 2 curves is filled. The
+                color of the first line and its alpha value are used for the
+                part filled. In depict, you cannot have more control about
+                this area
+
+            save_path (None, str): If None, the graph is not saved. If str,
+                the graph is saved in html at the gicen path. If the html
+                extension is missing, it will be added automatically
+
+            grid_visible (bool): Whether the background grid must be displayed.
+                In depict, you cannot have further control about the background
+                grid
+
+        Returns:
+            None
+        """
         plot = line(y=y, x=x, source_dataframe=source_dataframe, width=width,
                     height=height, description=description, title=title,
                     x_label=x_label, y_label=y_label, show_plot=show_plot,
                     color=color, colorbar_type=colorbar_type, legend=legend,
                     line_width=line_width, alpha=alpha, style=style,
                     x_axis_type=x_axis_type, y_axis_type=y_axis_type,
-                    fill_between=fill_between, grid_visible=grid_visible,
-                    session=session, save_path=save_path)
+                    x_range=x_range, y_range=y_range, fill_between=fill_between,
+                    grid_visible=grid_visible, session=session,
+                    save_path=save_path)
         return plot
     return line_updated
